@@ -8,6 +8,7 @@ import pickle
 import pathlib
 import sys
 import filecmp
+import logging
 import io
 import os
 import re
@@ -21,15 +22,21 @@ from threading import Lock
 import matplotlib
 matplotlib.use('Agg')
 import matplotlib.pyplot as plt
-# import networkx as nx
+
 import numpy
+
 from deap import base
 from deap import creator
 from deap import tools
+
 import pptx
 import signal, psutil
-# from tqdm import tqdm
+
 from rich import print
+import rich.progress
+from rich.progress import Progress
+from rich.logging import RichHandler
+
 import pycuda.driver as cuda
 
 from gevo import irind
@@ -74,7 +81,9 @@ class evolution:
         def evaluate(self):
             # Since golden has been filled up, passing this testcase into resultcompare
             # won't compare anything which is exactly what we want.
-            self.fitness = self._evolution.execNVprofRetrive(self)
+            # Evaluate 3 times and get the minimum number
+            fitness = [ self._evolution.execNVprofRetrive(self) for i in range(3)]
+            self.fitness = (min([ value[0] for value in fitness ]), min([ value[1] for value in fitness ]))
             if None in self.fitness:
                 print(self.args)
                 raise Exception("Original binary execution error")
@@ -93,7 +102,6 @@ class evolution:
         self.err_rate = err_rate
         self.kernels = kernel
         self.appBinary = bin
-        # self.appArgs = "" if args is None else args
         self.timeout = timeout
         self.fitness_function = fitness
         self.use_fitness_map = use_fitness_map
@@ -160,18 +168,24 @@ class evolution:
         self.testcase = []
         for i in range(len(arg_array)):
             self.testcase.append( self._testcase(self, i, kernel, bin, profile['verify']) )
-        print("evalute testcase as golden..", end='', flush=True)
-        for i, (tc, arg) in enumerate(zip(self.testcase, arg_array)):
-            tc.args = arg
-            print("{}..".format(i+1), end='', flush=True)
-            tc.evaluate()
-        print("done", flush=True)
+        print("Evalute testcase as golden..", end='', flush=True)
+        with Progress("[Initializing GEVO] Evaluate original program with test cases",
+                      "({task.completed} / {task.total})",
+                      rich.progress.TimeElapsedColumn()) as progress:
+            task = progress.add_task("", total=len(arg_array))
+            for tc, arg in zip(self.testcase, arg_array):
+                tc.args = arg
+                tc.evaluate()
+                progress.update(task, advance=1)
 
-        fits = [ tc.fitness[0] for tc in self.testcase]
-        errs = [ tc.fitness[1] for tc in self.testcase]
-        self.origin.fitness.values = (sum(fits)/len(fits), max(errs))
+        self.ofits = [ tc.fitness[0] for tc in self.testcase]
+        self.oerrs = [ tc.fitness[1] for tc in self.testcase]
+        self.origin.fitness.values = (sum(self.ofits)/len(self.ofits), max(self.oerrs))
         self.editFitMap[None] = self.origin.fitness.values
-        print("Fitness of the original program: {}".format(self.origin.fitness.values))
+        print(f"Average fitness of the original program: ({self.origin.fitness.values[0]:.2f}, {self.origin.fitness.values[1]:.2f})")
+        print("Individual test cases:")
+        for fit, err in zip(self.ofits, self.oerrs):
+            print(f"\t({fit:.2f}, {err:.2f})")
 
     def updateSlideFromPlot(self):
         pffits = [ind.fitness.values for ind in self.paretof]
@@ -407,7 +421,7 @@ class evolution:
             # Unknown nvprof error
             if retcode != 0:
                 print(stderr.decode(), file=sys.stderr)
-                raise Exception('nvprof error')
+                raise Exception('Unknown nvprof error code:{}'.format(retcode))
         except subprocess.TimeoutExpired:
             # Sometimes terminating nvprof will not terminate the underlying cuda program
             # if that program is corrupted. So issue the kill command to those cuda app first
@@ -542,6 +556,7 @@ class evolution:
                 stage = json.load(open(stageFileName))
                 allEdits = [entry['edits'] for entry in stage]
             except:
+                print(f"GEVO Error in loading stage file \"{stageFileName}\"")
                 print(sys.exc_info())
                 exit(1)
 
